package de.fhdw.wtf.parser.testWithScanner;

import static org.junit.Assert.assertEquals;

import java.io.IOException;

import de.fhdw.wtf.common.ast.Model;
import de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException;
import de.fhdw.wtf.common.stream.FilteredTokenStream;
import de.fhdw.wtf.common.stream.ScannerInputStream;
import de.fhdw.wtf.dsl.scanner.test.VerboseTokenStream;
import de.fhdw.wtf.parser.Parser;
import de.fhdw.wtf.testutil.TestUtil;

@SuppressWarnings("javadoc")
public class TestPartslist {
	
	private static final String pathToTestFileDir = "src/test/java/de/fhdw/wtf/parser/test/files/";
	private static final String modelSuffix = "model";
	
	// TODO Testfall darf nicht auf ToString basieren! Bug #677
	
	private static final String expected =
			"Model: \n"
					+ "core:group=[ \n"
					+ "core>server:class={ \n"
					+ "components:Component*; \n"
					+ "manager:PartsListManager; \n"
					+ "clearComponents:[()->{}]; \n"
					+ "findComponent:[(name:String)->{}]; \n"
					+ "createComponent:[(name:String,price:Integer,componentType:String)->{exceptionGroup>DoubleDefinitionException}]; \n"
					+ "addComponent:[(product:Product,newComp:Component,amount:Integer)->{exceptionGroup>CycleException}]; \n"
					+ "changePrice:[(component:Component,newPrice:Integer)->{}]; \n"
					+ "getMaterialList:[(component:Component)->util>ListMaterial]; \n"
					+ "getOverallPrice:[(component:Component)->Integer]; \n"
					+ "count:[(component:Component)->Integer]; \n"
					+ "}; \n"
					+ "core>PartsListManager:class={ \n"
					+ "createComponent:[(name:String,price:Integer,componentType:String)->{exceptionGroup>DoubleDefinitionException}]; \n"
					+ "addComponent:[(product:Product,newComp:Component,amount:Integer)->{exceptionGroup>CycleException}]; \n"
					+ "changePrice:[(component:Component,newPrice:Integer)->{}]; \n"
					+ "getMaterialList:[(component:Component)->util>ListMaterial]; \n"
					+ "getOverallPrice:[(component:Component)->Integer]; \n"
					+ "count:[(component:Component)->Integer]; \n" + "}; \n" + "core>Component:class={ \n"
					+ "name:String prior; \n" + "price:Integer prior; \n" + "state:state>State prior; \n"
					+ "fetchMaterialList:[()->util>ListMaterial] abstract; \n"
					+ "fetchOverallPrice:[()->Integer] abstract; \n" + "changePrice:[(newPrice:Integer)->{}]; \n"
					+ "update:[(e:event>AbstractEvent)->{}] abstract; \n"
					+ "notifyObservingProducts:[(e:event>AbstractEvent)->{}] abstract; \n"
					+ "contains:[(component:Component)->{util>UserTrue,util>UserFalse}] abstract; \n"
					+ "getComponentInfo:[()->(name:String,price:Integer)]; \n" + "} abstract; \n"
					+ "core>Material:class=Component + { \n" + "}; \n" + "core>Product:class=Component + { \n"
					+ "parts:QPart*; \n"
					+ "addComponent:[(newComponent:Component,amount:Integer)->{exceptionGroup>CycleException}]; \n"
					+ "}; \n" + "core>QPart:class={ \n" + "amount:Integer prior; \n" + "component:Component prior; \n"
					+ "}; \n" + "core>util:group=[ \n" + "core>util>ListMaterial:class={ \n" + "entries:QMaterial*; \n"
					+ "addML:[(other:ListMaterial)->ListMaterial]; \n"
					+ "mulScalar:[(factor:Integer)->ListMaterial]; \n" + "}; \n" + "core>util>QMaterial:class={ \n"
					+ "amount:Integer; \n" + "material:Material; \n" + "}; \n" + "core>util>UserTrue:class={ \n"
					+ "}; \n" + "core>util>UserFalse:class={ \n" + "}; \n" + "]; \n" + "core>event:group=[ \n"
					+ "core>event>AbstractEvent:class={ \n" + "} abstract; \n"
					+ "core>event>StructureEvent:class=AbstractEvent + { \n" + "}; \n"
					+ "core>event>PriceEvent:class=AbstractEvent + { \n" + "}; \n" + "]; \n" + "core>state:group=[ \n"
					+ "core>state>State:class={ \n" + "} abstract; \n" + "core>state>NotCached:class=State + { \n"
					+ "}; \n" + "core>state>PriceCached:class=State + { \n" + "price:Integer prior; \n" + "}; \n"
					+ "core>state>StateAbstract:class=State + { \n" + "materials:ListMaterial prior; \n"
					+ "} abstract; \n" + "core>state>MaterialCached:class=StateAbstract + { \n" + "}; \n"
					+ "core>state>MaterialPriceCached:class=StateAbstract + { \n" + "price:Integer prior; \n" + "}; \n"
					+ "]; \n" + "core>exceptionGroup:group=[ \n"
					+ "core>exceptionGroup>DoubleDefinitionException:exception={ \n" + "}; \n"
					+ "core>exceptionGroup>CycleException:exception={ \n" + "}; \n" + "]; \n" + "]; \n";
	
	public void modelPartslist() throws IOException {
		final ScannerInputStream inputFile =
				TestUtil.readFileToScannerStream(pathToTestFileDir + "partslist" + "." + modelSuffix);
		final FilteredTokenStream output = FilteredTokenStream.create();
		final VerboseTokenStream v = new VerboseTokenStream(output);
		final de.fhdw.wtf.dsl.scanner.modelScanner.ModelDslScanner scanner =
				de.fhdw.wtf.dsl.scanner.modelScanner.ModelDslScanner.create();
		scanner.scan(inputFile, v);
		final Parser p = Parser.create(output);
		
		try {
			final Model actual = p.parse();
			assertEquals(expected, actual.toString());
		} catch (final NoValidTokenStreamException e) {
			// Nothing
		}
		
		assertEquals(0, p.getExceptions().size());
	}
	
}
